#  Copyright 2014 Klaudiusz Staniek
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""
 fiblary.rooms
 ~~~~~~~~~~~~~~

 Home Center Room Manager Implementation
"""

from fiblary3.client.v4 import base


class Controller(base.CommonController):
    RESOURCE = 'rooms'
    API_PARAMS = ('id', 'sectionID')



#  Copyright 2014 Klaudiusz Staniek
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""
 fiblary.timiestamp
 ~~~~~~~~~~~~~~~~~~

 Timestamp manipulations

"""

import datetime as dt
import dateutil


def timestamp_to_iso(timestamp):
    return dt.datetime.fromtimestamp(timestamp).isoformat(' ')


def datetime_to_epoch(date_time):
    return date_time.strftime('%s')
    # return int((date_time - dt.datetime(1970,1,1)).total_seconds())


def string_to_datetime(date_time):
    return dateutil.parser.parse(date_time)



#   Copyright 2013 Nebula Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""REST API bits"""

import json
import logging
import requests

from fiblary3.common import exceptions

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib.parse import urlencode


USER_AGENT = 'RAPI'

_logger = logging.getLogger(__name__)


class RESTApi(object):
    """A REST API client that handles the interface from us to the server

    RESTApi is requests.Session wrapper that knows how to do:
    * JSON serialization/deserialization
    * log requests in 'curl' format
    * basic API boilerplate for create/delete/list/set/show verbs

    * authentication is handled elsewhere and a token is passed in

    The expectation that there will be a RESTApi object per authentication
    token in use, i.e. project/username/auth_endpoint

    On the other hand, a Client knows details about the specific REST Api that
    it communicates with, such as the available endpoints, API versions, etc.
    """

    def __init__(
        self,
        session=None,
        username=None,
        password=None,
        user_agent=USER_AGENT,
        verify=True,
        logger=None,
        debug=None,
        base_url=None,
    ):
        """Construct a new REST client

        :param object session: A Session object to be used for
                               communicating with the identity service.
        :param string auth_header: A token from an initialized auth_reference
                                   to be used in the X-Auth-Token header
        :param string user_agent: Set the User-Agent header in the requests
        :param boolean/string verify: If ``True``, the SSL cert will be
                                      verified. A CA_BUNDLE path can also be
                                      provided.
        :param logging.Logger logger: A logger to output to. (optional)
        :param boolean debug: Enables debug logging of all request and
                              responses to identity service.
                              default False (optional)
        """

        if username and password:
            self.set_auth((username, password))
        else:
            self.set_auth(None)

        self.debug = debug
        self.base_url = base_url or ""

        if not session:
            # We create a default session object
            session = requests.Session()
        self.session = session
        self.session.verify = verify
        self.session.headers['User-Agent'] = user_agent
        self.session.stream = False

        if logger:
            self.logger = logger
        else:
            self.logger = _logger

    def set_auth(self, auth_header):
        """Sets the current auth blob"""
        self.auth_header = auth_header

    def set_header(self, header, content):
        """Sets passed in headers into the session headers

        Replaces existing headers!!
        """
        if content is None:
            del self.session.headers[header]
        else:
            self.session.headers[header] = content

    def request(self, method, url, **kwargs):
        """Make an authenticated (if token available) request

        :param method: Request HTTP method
        :param url: Request URL
        :param data: Request body
        :param json: Request body to be encoded as JSON
                     Overwrites ``data`` argument if present
        """

        kwargs.setdefault('headers', {})
        if self.auth_header:
            kwargs['auth'] = self.auth_header

        if 'json' in kwargs:
            json_obj = kwargs.get('json', None)
            if json_obj and isinstance(json_obj, (dict)):
                kwargs['data'] = json.dumps(json_obj)
                kwargs['headers']['Content-Type'] = 'application/json'
            kwargs.pop('json')

        kwargs.setdefault('allow_redirects', True)

        if 'timeout' not in kwargs:
            kwargs['timeout'] = 10

        if self.debug:
            self._log_request(method, self.base_url + url, **kwargs)

        try:
            response = self.session.request(method,
                                            self.base_url + url,
                                            **kwargs)
        except (requests.exceptions.ConnectionError,
                requests.exceptions.Timeout) as e:
            raise exceptions.ConnectionError(e.message)

        except Exception as e:
            raise e

        if self.debug:
            self._log_response(response)

        return self._error_handler(response)

    def _error_handler(self, response):
        if response.status_code < 200 or response.status_code > 300:
            self.logger.debug(
                "ERROR: %s",
                response.text,
            )
            raise exceptions.from_response(response)
        return response

    # Convenience methods to mimic the ones provided by requests.Session

    def delete(self, url, **kwargs):
        """Send a DELETE request. Returns :class:`requests.Response` object.

        :param url: Request URL
        :param \*\*kwargs: Optional arguments passed to ``request``
        """

        return self.request('DELETE', url, **kwargs)

    def get(self, url, **kwargs):
        """Send a GET request. Returns :class:`requests.Response` object.

        :param url: Request URL
        :param \*\*kwargs: Optional arguments passed to ``request``
        """

        return self.request('GET', url, **kwargs)

    def head(self, url, **kwargs):
        """Send a HEAD request. Returns :class:`requests.Response` object.

        :param url: Request URL
        :param \*\*kwargs: Optional arguments passed to ``request``
        """

        kwargs.setdefault('allow_redirects', False)
        return self.request('HEAD', url, **kwargs)

    def options(self, url, **kwargs):
        """Send an OPTIONS request. Returns :class:`requests.Response` object.

        :param url: Request URL
        :param \*\*kwargs: Optional arguments passed to ``request``
        """

        return self.request('OPTIONS', url, **kwargs)

    def patch(self, url, data=None, json=None, **kwargs):
        """Send a PUT request. Returns :class:`requests.Response` object.

        :param url: Request URL
        :param data: Request body
        :param json: Request body to be encoded as JSON
                     Overwrites ``data`` argument if present
        :param \*\*kwargs: Optional arguments passed to ``request``
        """

        return self.request('PATCH', url, data=data, json=json, **kwargs)

    def post(self, url, data=None, json=None, **kwargs):
        """Send a POST request. Returns :class:`requests.Response` object.

        :param url: Request URL
        :param data: Request body
        :param json: Request body to be encoded as JSON
                     Overwrites ``data`` argument if present
        :param \*\*kwargs: Optional arguments passed to ``request``
        """

        return self.request('POST', url, data=data, json=json, **kwargs)

    def put(self, url, data=None, json=None, **kwargs):
        """Send a PUT request. Returns :class:`requests.Response` object.

        :param url: Request URL
        :param data: Request body
        :param json: Request body to be encoded as JSON
                     Overwrites ``data`` argument if present
        :param \*\*kwargs: Optional arguments passed to ``request``
        """

        return self.request('PUT', url, data=data, json=json, **kwargs)

    # Command verb methods

    def create(self, url, data=None, response_key=None, **kwargs):
        """Create a new object via a POST request

        :param url: Request URL
        :param data: Request body, will be JSON encoded
        :param response_key: Dict key in response body to extract
        :param \*\*kwargs: Optional arguments passed to ``request``
        """

        response = self.request('POST', url, json=data, **kwargs)
        if response_key:
            return response.json()[response_key]
        else:
            return response.json()

    def list(self, url, data=None, response_key=None, **kwargs):
        """Retrieve a list of objects via a GET or POST request

        :param url: Request URL
        :param data: Request body, will be JSON encoded
        :param response_key: Dict key in response body to extract
        :param \*\*kwargs: Optional arguments passed to ``request``
        """

        if data:
            response = self.request('POST', url, json=data, **kwargs)
        else:
            response = self.request('GET', url, **kwargs)

        if response_key:
            return response.json()[response_key]
        else:
            return response.json()

    def set(self, url, data=None, response_key=None, **kwargs):
        """Update an object via a PUT request

        :param url: Request URL
        :param data: Request body
        :param json: Request body to be encoded as JSON
                     Overwrites ``data`` argument if present
        :param \*\*kwargs: Optional arguments passed to ``request``
        """

        response = self.request('PUT', url, json=data)
        if data:
            if response_key:
                return response.json()[response_key]
            else:
                return response.json()
        else:
            # Nothing to do here
            return None

    def show(self, url, response_key=None, **kwargs):
        """Retrieve a single object via a GET request

        :param url: Request URL
        :param response_key: Dict key in response body to extract
        :param \*\*kwargs: Optional arguments passed to ``request``
        """

        response = self.request('GET', url, **kwargs)
        if response_key:
            return response.json()[response_key]
        else:
            return response.json()

    def _log_request(self, method, url, **kwargs):
        if 'params' in kwargs and kwargs['params'] != {}:
            url += '?' + urlencode(kwargs['params'])

        string_parts = [
            "curl -i",
            "-X '%s'" % method,
            "'%s'" % url,
        ]

        for element in kwargs['headers']:
            header = " -H '%s: %s'" % (element, kwargs['headers'][element])
            string_parts.append(header)

        self.logger.debug("REQ: %s" % " ".join(string_parts))
        if 'data' in kwargs:
            self.logger.debug("  REQ BODY: %r\n" % (kwargs['data']))

    def _log_response(self, response):
        self.logger.debug(
            "RESP: [%s] %r",
            response.status_code,
            response.headers,
        )
        if response._content_consumed:
            self.logger.debug(
                "  RESP BODY: '%s'",
                response.text,
            )
        self.logger.debug(
            "  encoding: %s",
            response.encoding,
        )



#  Copyright 2014 Klaudiusz Staniek
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""
 fiblary.models
 ~~~~~~~~~~~~~~~~~

 Home Center Model Class Implementations

"""

import functools
import jsonpatch
import logging
import six

_logger = logging.getLogger(__name__)


_type_ignore = ["HC_user", "VOIP_user", "weather", 'iOS_device', '']


def factory(controller, item):
    # try as item could be anything
    try:
        if item['type'] in _type_ignore:
            return None
    except Exception:
        pass

    model = None
    if isinstance(item, dict):
        if controller.RESOURCE == 'devices':
            model = DeviceModel(controller, item)
        elif controller.RESOURCE == 'scenes':
            model = SceneModel(controller, item)
        else:
            model = GenericModel(controller, item)
    elif isinstance(item, list):
        model = RecursiveList(item)
    else:
        assert 0, "Unknown model"
    return model


class RecursiveList(list):

    def __init__(self, value):
        if value is None:
            pass
        elif isinstance(value, list):
            for index, data in enumerate(value):
                self.append(data)
                self.__setitem__(index, data)
        else:
            raise TypeError('Expected list')

        self.__dict__['__original__'] = value

    def __getitem__(self, key):
        return list.__getitem__(self, key)

    def __setitem__(self, key, value):
        if isinstance(value, str):
            value = str(value)
        if isinstance(value, list) and not isinstance(value, RecursiveList):
            value = RecursiveList(value)

        if isinstance(value, dict) and not isinstance(value, RecursiveDict):
            value = RecursiveDict(value)

        list.__setitem__(self, key, value)

    __setattr__ = __setitem__
    __getattr__ = __getitem__


class RecursiveDict(dict):
    def __init__(self, value=None):
        if value is None:
            pass
        elif isinstance(value, dict):
            for key in value:
                self.__setitem__(key, value[key])
        else:
            raise TypeError('Expected dict')
        self.__dict__['__original__'] = value

    def changes(self):
        original = self.__dict__['__original__']
        return jsonpatch.make_patch(original, dict(self)).to_string()

    def __setitem__(self, key, value):
        if isinstance(value, str):
            value = str(value)
        if isinstance(value, dict) and not isinstance(value, RecursiveDict):
            value = RecursiveDict(value)

        if isinstance(value, list) and not isinstance(value, RecursiveList):
            value = RecursiveList(value)

        if not callable(value):
            dict.__setitem__(self, key, value)
        else:
            # actions are callable so added only to the local dict
            self.__dict__[key] = value

    def __getitem__(self, key):
        return dict.__getitem__(self, key)

    def setdefault(self, key, default=None):
        if key not in self:
            self[key] = default
        return self[key]

    __setattr__ = __setitem__
    __getattr__ = __getitem__


class GenericModel(RecursiveDict):
    def __init__(self, controller, item):
        self.__dict__['controller'] = controller
        super(GenericModel, self).__init__(item)


class DeviceModel(GenericModel):
    def __init__(self, controller, item):
        super(DeviceModel, self).__init__(controller, item)

        if 'actions' in self:
            def action(action_name, argn, *args, **kwargs):
                _logger.info("{0}({1})->{2}{3}".format(
                    self.name, self.id, action_name, args)
                )
                if len(args) != argn:
                    # hack due to http://bugzilla.fibaro.com/view.php?id=1125
                    if action_name != 'setTargetLevel':
                        raise TypeError(
                            "%s() takes exactly %d argument(s) (%d given)" % (
                                action_name, argn, len(args))
                        )
                return self.controller.action(self.id, action_name, *args)

            for action_name, argn in six.iteritems(self['actions']):
                _logger.debug("{0}({1})<-{2}({3})".format(
                    self.name,
                    self.id,
                    action_name,
                    argn))

                self.__dict__[str(action_name)] = functools.partial(
                    action, action_name, argn)


class SceneModel(GenericModel):
    def __init__(self, controller, item):
        super(SceneModel, self).__init__(controller, item)
    """Home Center specific subclass for the Scene with actions"""

    def start(self):
        return self.controller.start(self.id)

    def stop(self):
        return self.controller.stop(self.id)

    def enable(self):
        return self.controller.enable(self.id)

    def disable(self):
        return self.controller.disable(self.id)



#  Copyright 2014 Klaudiusz Staniek
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""
 fiblary.client
 ~~~~~~~~~~~~~~

 Home Center Controller Client Implementation

"""
import logging
import sys
import threading

from fiblary3.client.v4 import devices
from fiblary3.client.v4 import events
from fiblary3.client.v4 import info
from fiblary3.client.v4 import login
from fiblary3.client.v4 import models
from fiblary3.client.v4 import rooms
from fiblary3.client.v4 import scenes
from fiblary3.client.v4 import sections
from fiblary3.client.v4 import users
from fiblary3.client.v4 import variables
from fiblary3.client.v4 import weather

from fiblary3.common.event import EventHook
from fiblary3.common import exceptions
from fiblary3.common import restapi


_logger = logging.getLogger(__name__)

_schema_ignore = ["HC_user", "VOIP_user", "weather", 'iOS_device', '']

class Client(object):
    """Home Center 2 Client Class.
    Provides interface to different resources managed by HC2
    through the specialized controllers
    """
    def __init__(self, endpoint, username=None, password=None):

        if '/api/' not in endpoint:
            raise IOError("Wrong API string. It should be like: "
                          "http://<hc2_ip>/api/")

        # TODO(klstanie):  Add the HC2 reachability checking

        self.client = restapi.RESTApi(
            base_url=endpoint,
            username=username,
            password=password,
            debug=False
        )
        self.client.set_header("Fibaro Header","X-Fibaro-Version:2")

        self.modified = {}
        self.modified_lock = threading.Lock()

        # initialize the managers
        self.info = info.Controller(
            self.client,
            self._get_info_model()
        )

        self.login = login.Controller(
            self.client,
            self._get_login_model()
        )

        self.sections = sections.Controller(
            self.client,
            self._get_section_model()
        )

        self.rooms = rooms.Controller(
            self.client,
            self._get_room_model()
        )

        self.users = users.Controller(
            self.client,
            self._get_user_model()
        )

        self.variables = variables.Controller(
            self.client,
            self._get_variable_model()
        )

        self.scenes = scenes.Controller(
            self.client,
            self._get_scene_model()
        )

        self.devices = devices.Controller(
            self.client,
            self._get_device_model()
        )

        self.weather = weather.Controller(
            self.client,
            self._get_weather_model()
        )

        self.events = events.Controller(
            self.client,
            self._get_event_model()
        )

        self.state_handler = None

    def _get_info_model(self):
        def model(item):
            return models.factory(self.info, item)
        return model

    def _get_login_model(self):
        def model(item):
            return models.factory(self.login, item)
        return model

    def _get_section_model(self):
        def model(item):
            return models.factory(self.sections, item)
        return model

    def _get_room_model(self):
        def model(item):
            return models.factory(self.rooms, item)
        return model

    def _get_user_model(self):
        def model(item):
            return models.factory(self.users, item)
        return model

    def _get_variable_model(self):
        def model(item):
            return models.factory(self.variables, item)
        return model

    def _get_weather_model(self):
        def model(item):
            return models.factory(self.weather, item)
        return model

    def _get_event_model(self):
        def model(item):
            return models.factory(self.events, item)
        return model

    def _get_scene_model(self):
        def model(item):
            return models.factory(self.scenes, item)
        return model

    def _get_device_model(self):
        def model(item):
            return models.factory(self.devices, item)
        return model

    def __repr__(self):
        return "Home Center 2 Client"

    def _on_property_change(self, **kwargs):
        property_name = kwargs.get('property', None)
        if not property_name:
            return
        try:
            self.modified[property_name](**kwargs)
        except Exception:
            # trick to reduce number of exceptions
            with self.modified_lock:
                self.modified[property_name] = EventHook(property_name)
            _logger.warning("Missing state handler for: {}".format(property_name))

    def _on_state_change(self, state):
        timestamp = state.get('timestamp', 0)
        for change in state.get('changes', []):
            device_id = change.pop('id')
            for property_name, value in list(change.items()):
                data = {
                    'timestamp': timestamp,
                    'id': device_id,
                    'property': property_name,
                    'value': value,
                    'client': self
                }
                self._on_property_change(**data)

# API

    def enable_state_handler(self):
        self.state_handler = StateHandler(self, self._on_state_change)

    def disable_state_handler(self):
        self.state_handler.stop()

    def add_event_handler(self, property_name, handler):
        if property_name not in self.modified:
            with self.modified_lock:
                self.modified[property_name] = EventHook(property_name)

        self.modified[property_name] += handler

    def remove_event_handler(self, property_name, handler):
        try:
            self.modified[property_name] -= handler
        except ValueError:
            raise exceptions.HandlerNotFound(
                message="Handler for property '{}' not found: {}.".format(
                    property_name,
                    handler.__name__))


class StateHandler(threading.Thread):
    def __init__(self, client, callback):
        super(StateHandler, self).__init__(name=self.__class__.__name__)
        self.client = client
        self.api = client.client
        self.callback = callback
        self.daemon = True  # stop unconditionally on exit

        self._stop = threading.Event()

        self.start()

    def run(self):
        """State Handler main loop"""

        _logger.info("Starting the state change handler")
        last = "0"
        while not self.stopped():

            timeout = 60
            sleep_time = 1
            attempt = 1
            success = False

            while not success:
                if self.stopped():
                    break

                try:
                    state = self.api.get(
                        'refreshStates?last={}'.format(last),
                        timeout=timeout)
                    _logger.debug(state)
                    try:
                        state = state.json()
                    except Exception:
                        _logger.critical("JSON ERROR: {}".format(state))
                        raise
                    last = state['last']
                    self.callback(state)
                    success = True
                    break

                except Exception:
                    _logger.warn(
                        "Connection Error. Attempt number: {}".format(
                            attempt)
                    )
                    e = sys.exc_info()
                    _logger.exception("Exception: {}".format(str(e)))

                    attempt += 1

                if not success:
                    if attempt == 10:
                        sleep_time = 30
                        _logger.warn(
                            "Fallback to 30-second connection retry timer."
                        )

                    _logger.warn(
                        "Waiting for next attempt {} second(s)".format(
                            sleep_time)
                    )
                    self._stop.wait(sleep_time)

        _logger.info("State change handler stopped.")

    def stopped(self):
        return self._stop.isSet()

    def stop(self):
        _logger.info("Stopping the state change handler")

        self.api.session.close()  # not effect on pending request
        self._stop.set()



#  Copyright 2014 Klaudiusz Staniek
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""
 fiblary.scenes
 ~~~~~~~~~~~~~~

 Home Center Scene Manager Implementation
"""

import logging

from fiblary3.client.v3 import base
from fiblary3.common import exceptions


_logger = logging.getLogger(__name__)


class Controller(base.CommonController):
    RESOURCE = 'scenes'

    def _scene_control(self, scene_id, action):
        cmd = "sceneControl"
        params = {
            "id": scene_id,
            "action": action
        }
        resp = self.http_client.get(cmd, params=params)
        if resp.status_code != 202:
            exceptions.from_response(resp)

        return self.get(scene_id)

    def start(self, scene_id):
        scene = self._scene_control(scene_id, "start")
        _logger.info("Scene {}({}) started".format(scene.name, scene.id))
        return scene

    def stop(self, scene_id):
        scene = self._scene_control(scene_id, "stop")
        _logger.info("Scene {}({}) stopped".format(scene.name, scene.id))
        return scene

    def enable(self, scene_id):
        scene = self._scene_control(scene_id, "enable")
        _logger.info("Scene {}({}) enabled".format(scene.name, scene.id))
        return scene

    def disable(self, scene_id):
        scene = self._scene_control(scene_id, "disable")
        _logger.info("Scene {}({}) disabled".format(scene.name, scene.id))
        return scene

    def update(self, data):
        # scene is Null due to
        # http://bugzilla.fibaro.com/view.php?id=1176
        # must be retrieved again
        scene = super(Controller, self).update(data)
        try:
            scene_id = data['id']
            if scene_id:
                _logger.warning(
                    "Refer to: http://bugzilla.fibaro.com/view.php?id=1176")
                scene = self.get(scene_id)
        except Exception:
            pass

        return scene



#  Copyright 2014 Klaudiusz Staniek
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""
This is a fiblary package providing python API implementation to Home Center

"""


from fiblary3.common import utils


def Client(version, *args, **kwargs):
    """This is a client wrapper handling API versioning

    :param version: A version string of the API (i.e. 'v3')
    :returns: A Home Center Client class
    """

    module = utils.import_versioned_module('client', version, 'client')
    client_class = getattr(module, 'Client')
    return client_class(*args, **kwargs)



#!/usr/bin/env python3.5
#  Copyright 2014 Klaudiusz Staniek
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

import os
import sys

import fiblary3

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

packages = [
    'fiblary3',
    'fiblary3.client',
    'fiblary3.client.v3',
    'fiblary3.client.v4',
    'fiblary3.client.v5',
    'fiblary3.common',
    'fiblary3.external',
]

with open('requirements.txt') as f:
    requires = f.readlines()

with open('README.rst') as f:
    readme = f.read()

setup(
    name='fiblary3-fork',
    version=fiblary3.__version__,
    description='Home Center API Python Library - forked version',
    long_description=readme,
    author='Peter Balogh, Graham Bennett',
    author_email='graham@grahambennett.org',
    url='https://github.com/graham33/fiblary',
    packages=packages,
    package_data={'': ['LICENSE', ], },
    package_dir={'fiblary3': 'fiblary3'},
    include_package_data=True,
    install_requires=requires,
    license='Apache 2.0',
    zip_safe=False,
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ),
)



#  Copyright 2014 Klaudiusz Staniek
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""
Exceptions
"""

import sys


class CommandError(Exception):
    pass


class BaseException(Exception):
    """An error occurred."""
    def __init__(self, message=None):
        self.message = message

    def __str__(self):
        return self.message or self.__class__.__doc__


class ConnectionError(BaseException):
    """Connection Error"""
    pass


class NoUniqueMatch(BaseException):
    """No uniqueue item match"""
    pass


class NotFound(BaseException):
    """Item not found"""
    pass


class HandlerNotFound(BaseException):
    """Handler not found"""
    pass


class WrongArgumentsNumber(Exception):
    def __init__(self, action, expected_num_arg, num_arg):
            self.action = action
            self.num_arg = num_arg
            self.expected_num_arg = expected_num_arg

    def __str__(self):
        return "{0} takes exactly {1} arguments ({2} given)".format(
            repr(self.action),
            self.expected_num_arg,
            self.num_arg)


class HTTPException(Exception):
    """Base exception for all HTTP-derived exceptions."""
    code = 'N/A'

    def __init__(self, details=None):
        self.details = details or self.__class__.__doc__

    def __str__(self):
        return "%s (HTTP %s)" % (self.details, self.code)


class HTTPNotFound(HTTPException):
    """Not Found"""
    code = 404


class HTTPInternalServerError(HTTPException):
    """Internal Server Error"""
    code = 500


class HTTPMultipleChoices(HTTPException):
    """Multiple Choices"""
    code = 300


class HTTPBadRequest(HTTPException):
    """Bad Request"""
    code = 400

_code_map = {}
for obj_name in dir(sys.modules[__name__]):
    if obj_name.startswith('HTTP'):
        obj = getattr(sys.modules[__name__], obj_name)
        _code_map[obj.code] = obj


def from_response(response, details=True):
    """Return an instance of an HTTPException based on requests response."""
    cls = _code_map.get(response.status_code, HTTPException)
    if details:
        details = response.reason
        return cls(details=details)

    return cls()
